@@ -43,6 +43,7 @@
 #include "nm-setting-ip-tunnel.h"
 #include "nm-setting-ip4-config.h"
 #include "nm-setting-ip6-config.h"
+#include "nm-setting-link.h"
 #include "nm-setting-loopback.h"
 #include "nm-setting-macsec.h"
 #include "nm-setting-macvlan.h"
@@ -322,8 +323,6 @@ typedef gpointer (*NMUtilsCopyFunc)(gpointer);
 const char **
 _nm_ip_address_get_attribute_names(const NMIPAddress *addr, gboolean sorted, guint *out_length);
 
-char *nm_setting_infiniband_create_virtual_interface_name(const char *parent, int p_key);
-
 #define NM_SETTING_WIRED_S390_OPTION_MAX_LEN 200u
 
 void     _nm_setting_wired_clear_s390_options(NMSettingWired *setting);
@@ -472,6 +471,7 @@ typedef enum {
     NM_BOND_OPTION_TYPE_INT,
     NM_BOND_OPTION_TYPE_BOTH,
     NM_BOND_OPTION_TYPE_IP,
+    NM_BOND_OPTION_TYPE_IP6,
     NM_BOND_OPTION_TYPE_MAC,
     NM_BOND_OPTION_TYPE_IFNAME,
 } NMBondOptionType;
@@ -1023,6 +1023,8 @@ _nm_connection_serialize_secrets(NMConnectionSerializationFlags flags,
 void _nm_connection_clear_secrets_by_secret_flags(NMConnection        *self,
                                                   NMSettingSecretFlags filter_flags);
 
+typedef gboolean (*_NMConnectionForEachSecretFunc)(NMSettingSecretFlags flags, gpointer user_data);
+
 GVariant *_nm_connection_for_each_secret(NMConnection                  *self,
                                          GVariant                      *secrets,
                                          gboolean                       remove_non_secrets,
