@@ -12,6 +12,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/un.h>
+#include <sys/ioctl.h>
 
 #include "nm-str-buf.h"
 #include "nm-shared-utils.h"
@@ -20,9 +21,9 @@
 
 /*****************************************************************************/
 
-_nm_printf(4, 5) static int _get_contents_error(GError **   error,
+_nm_printf(4, 5) static int _get_contents_error(GError    **error,
                                                 int         errsv,
-                                                int *       out_errsv,
+                                                int        *out_errsv,
                                                 const char *format,
                                                 ...)
 {
@@ -94,14 +95,14 @@ nm_utils_fd_get_contents(int                         fd,
                          gboolean                    close_fd,
                          gsize                       max_length,
                          NMUtilsFileGetContentsFlags flags,
-                         char **                     contents,
-                         gsize *                     length,
-                         int *                       out_errsv,
-                         GError **                   error)
+                         char                      **contents,
+                         gsize                      *length,
+                         int                        *out_errsv,
+                         GError                    **error)
 {
     nm_auto_close int fd_keeper = close_fd ? fd : -1;
     struct stat       stat_buf;
-    gs_free char *    str          = NULL;
+    gs_free char     *str          = NULL;
     const bool        do_bzero_mem = NM_FLAGS_HAS(flags, NM_UTILS_FILE_GET_CONTENTS_FLAG_SECRET);
     int               errsv;
 
@@ -288,13 +289,13 @@ nm_utils_fd_get_contents(int                         fd,
  */
 gboolean
 nm_utils_file_get_contents(int                         dirfd,
-                           const char *                filename,
+                           const char                 *filename,
                            gsize                       max_length,
                            NMUtilsFileGetContentsFlags flags,
-                           char **                     contents,
-                           gsize *                     length,
-                           int *                       out_errsv,
-                           GError **                   error)
+                           char                      **contents,
+                           gsize                      *length,
+                           int                        *out_errsv,
+                           GError                    **error)
 {
     int fd;
 
@@ -338,19 +339,20 @@ nm_utils_file_get_contents(int                         dirfd,
  * and last modification times.
  */
 gboolean
-nm_utils_file_set_contents(const char *           filename,
-                           const char *           contents,
+nm_utils_file_set_contents(const char            *filename,
+                           const char            *contents,
                            gssize                 length,
                            mode_t                 mode,
                            const struct timespec *times,
-                           int *                  out_errsv,
-                           GError **              error)
+                           int                   *out_errsv,
+                           GError               **error)
 {
     gs_free char *tmp_name = NULL;
     struct stat   statbuf;
     int           errsv;
     gssize        s;
     int           fd;
+    int           r;
 
     g_return_val_if_fail(filename, FALSE);
     g_return_val_if_fail(contents || !length, FALSE);
@@ -414,7 +416,16 @@ nm_utils_file_set_contents(const char *           filename,
                                    tmp_name);
     }
 
-    nm_close(fd);
+    r = nm_close_with_error(fd);
+    if (r < 0) {
+        errsv = NM_ERRNO_NATIVE(-r);
+        unlink(tmp_name);
+        return _get_contents_error(error,
+                                   errsv,
+                                   out_errsv,
+                                   "failed close() after writing file %s",
+                                   tmp_name);
+    }
 
     if (rename(tmp_name, filename)) {
         errsv = NM_ERRNO_NATIVE(errno);
@@ -471,13 +482,17 @@ nm_utils_fd_read(int fd, NMStrBuf *out_string)
     g_return_val_if_fail(fd >= 0, -1);
     g_return_val_if_fail(out_string, -1);
 
-    /* If the buffer size is 0, we allocate NM_UTILS_GET_NEXT_REALLOC_SIZE_1000 (1000 bytes)
-     * the first time. Afterwards, the buffer grows exponentially.
+    /* Reserve at least 488+1 bytes of buffer size. That is probably a suitable
+     * compromise between not wasting too much buffer space and not reading too much.
      *
-     * Note that with @buf_available, we always would read as much buffer as we actually
-     * have reserved. */
-    nm_str_buf_maybe_expand(out_string, NM_UTILS_GET_NEXT_REALLOC_SIZE_1000, FALSE);
+     * Note that when we start with an empty buffer, the first allocation of
+     * 488+1 bytes will actually allocate 1000 bytes. So if we were to receive
+     * one byte at a time, we don't need a reallocation for the first 1000-(488+1)
+     * bytes. Afterwards grows the buffer exponentially.
+     */
+    nm_str_buf_maybe_expand(out_string, NM_UTILS_GET_NEXT_REALLOC_SIZE_488 + 1, FALSE);
 
+    /* We always use all the available buffer size. */
     buf_available = out_string->allocated - out_string->len;
 
     n_read = read(fd, &((nm_str_buf_get_str_unsafe(out_string))[out_string->len]), buf_available);
@@ -496,9 +511,48 @@ nm_utils_fd_read(int fd, NMStrBuf *out_string)
 
 /*****************************************************************************/
 
+/* Taken from systemd's next_datagram_size_fd(). */
+gssize
+nm_fd_next_datagram_size(int fd)
+{
+    gssize l;
+    int    k;
+
+    /* This is a bit like FIONREAD/SIOCINQ, however a bit more powerful. The difference being: recv(MSG_PEEK) will
+     * actually cause the next datagram in the queue to be validated regarding checksums, which FIONREAD doesn't
+     * do. This difference is actually of major importance as we need to be sure that the size returned here
+     * actually matches what we will read with recvmsg() next, as otherwise we might end up allocating a buffer of
+     * the wrong size. */
+
+    l = recv(fd, NULL, 0, MSG_PEEK | MSG_TRUNC);
+    if (l < 0) {
+        if (NM_IN_SET(errno, EOPNOTSUPP, EFAULT))
+            goto fallback;
+
+        return -errno;
+    }
+    if (l == 0)
+        goto fallback;
+
+    return l;
+
+fallback:
+    k = 0;
+
+    /* Some sockets (AF_PACKET) do not support null-sized recv() with MSG_TRUNC set, let's fall back to FIONREAD
+     * for them. Checksums don't matter for raw sockets anyway, hence this should be fine. */
+
+    if (ioctl(fd, FIONREAD, &k) < 0)
+        return -errno;
+
+    return (gssize) k;
+}
+
+/*****************************************************************************/
+
 typedef struct {
     GSubprocess *subprocess;
-    GSource *    timeout_source;
+    GSource     *timeout_source;
 } SubprocessTerminateData;
 
 static void
@@ -527,7 +581,7 @@ void
 nm_g_subprocess_terminate_in_background(GSubprocess *subprocess, int timeout_msec_before_kill)
 {
     SubprocessTerminateData *term_data;
-    GMainContext *           main_context;
+    GMainContext            *main_context;
 
     nm_assert(timeout_msec_before_kill > 0);
 
@@ -573,9 +627,9 @@ char **
 nm_utils_find_mkstemp_files(const char *dirname, const char *filename)
 {
     static const char letters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
-    DIR *             dir;
-    struct dirent *   entry;
-    GPtrArray *       arr = NULL;
+    DIR              *dir;
+    struct dirent    *entry;
+    GPtrArray        *arr = NULL;
     gsize             l;
 
     /* We write files with g_file_set_contents() and nm_utils_file_set_contents().
@@ -685,12 +739,12 @@ nm_sd_notify(const char *state)
     struct sockaddr_un sockaddr;
     struct iovec       iovec;
     struct msghdr      msghdr = {
-        .msg_iov    = &iovec,
-        .msg_iovlen = 1,
-        .msg_name   = &sockaddr,
+             .msg_iov    = &iovec,
+             .msg_iovlen = 1,
+             .msg_name   = &sockaddr,
     };
     nm_auto_close int fd = -1;
-    const char *      e;
+    const char       *e;
     int               r;
 
     if (!state)
@@ -723,3 +777,288 @@ nm_sd_notify(const char *state)
 
     return 0;
 }
+
+/*****************************************************************************/
+
+#define SHELL_NEED_ESCAPE "\"\\`$"
+
+int
+nm_parse_env_file_full(
+    const char *contents,
+    int (*push)(unsigned line, const char *key, const char *value, void *userdata),
+    void *userdata)
+{
+    gsize                    last_value_whitespace = G_MAXSIZE;
+    gsize                    last_key_whitespace   = G_MAXSIZE;
+    nm_auto_str_buf NMStrBuf key                   = NM_STR_BUF_INIT(0, FALSE);
+    nm_auto_str_buf NMStrBuf value                 = NM_STR_BUF_INIT(0, FALSE);
+    unsigned                 line                  = 1;
+    int                      r;
+    enum {
+        PRE_KEY,
+        KEY,
+        PRE_VALUE,
+        VALUE,
+        VALUE_ESCAPE,
+        SINGLE_QUOTE_VALUE,
+        DOUBLE_QUOTE_VALUE,
+        DOUBLE_QUOTE_VALUE_ESCAPE,
+        COMMENT,
+        COMMENT_ESCAPE
+    } state = PRE_KEY;
+
+    /* Copied and adjusted from systemd's parse_env_file_internal().
+     * https://github.com/systemd/systemd/blob/6247128902ca71ee2ad406cf69af04ea389d3d27/src/basic/env-file.c#L15 */
+
+    nm_assert(push);
+
+    if (!contents)
+        return -ENOENT;
+
+    for (const char *p = contents; *p; p++) {
+        char c = *p;
+
+        switch (state) {
+        case PRE_KEY:
+            if (NM_IN_SET(c, '#', ';'))
+                state = COMMENT;
+            else if (!nm_ascii_is_whitespace(c)) {
+                state               = KEY;
+                last_key_whitespace = G_MAXSIZE;
+                nm_str_buf_append_c(&key, c);
+            }
+            break;
+
+        case KEY:
+            if (nm_ascii_is_newline(c)) {
+                state = PRE_KEY;
+                line++;
+                nm_str_buf_reset(&key);
+            } else if (c == '=') {
+                state                 = PRE_VALUE;
+                last_value_whitespace = G_MAXSIZE;
+            } else {
+                if (!nm_ascii_is_whitespace(c))
+                    last_key_whitespace = G_MAXSIZE;
+                else if (last_key_whitespace == G_MAXSIZE)
+                    last_key_whitespace = key.len;
+                nm_str_buf_append_c(&key, c);
+            }
+            break;
+
+        case PRE_VALUE:
+            if (nm_ascii_is_newline(c)) {
+                state = PRE_KEY;
+                line++;
+
+                /* strip trailing whitespace from key */
+                if (last_key_whitespace != G_MAXSIZE)
+                    nm_str_buf_get_str_unsafe(&key)[last_key_whitespace] = 0;
+
+                r = push(line,
+                         nm_str_buf_get_str(&key),
+                         nm_str_buf_get_str(&value) ?: "",
+                         userdata);
+                if (r < 0)
+                    return r;
+
+                nm_str_buf_reset(&key);
+                nm_str_buf_reset(&value);
+            } else if (c == '\'')
+                state = SINGLE_QUOTE_VALUE;
+            else if (c == '"')
+                state = DOUBLE_QUOTE_VALUE;
+            else if (c == '\\')
+                state = VALUE_ESCAPE;
+            else if (!nm_ascii_is_whitespace(c)) {
+                state = VALUE;
+                nm_str_buf_append_c(&value, c);
+            }
+
+            break;
+
+        case VALUE:
+            if (nm_ascii_is_newline(c)) {
+                state = PRE_KEY;
+                line++;
+
+                /* Chomp off trailing whitespace from value */
+                if (last_value_whitespace != G_MAXSIZE)
+                    nm_str_buf_get_str_unsafe(&value)[last_value_whitespace] = 0;
+
+                /* strip trailing whitespace from key */
+                if (last_key_whitespace != G_MAXSIZE)
+                    nm_str_buf_get_str_unsafe(&key)[last_key_whitespace] = 0;
+
+                r = push(line,
+                         nm_str_buf_get_str(&key),
+                         nm_str_buf_get_str(&value) ?: "",
+                         userdata);
+                if (r < 0)
+                    return r;
+
+                nm_str_buf_reset(&key);
+                nm_str_buf_reset(&value);
+            } else if (c == '\\') {
+                state                 = VALUE_ESCAPE;
+                last_value_whitespace = G_MAXSIZE;
+            } else {
+                if (!nm_ascii_is_whitespace(c))
+                    last_value_whitespace = G_MAXSIZE;
+                else if (last_value_whitespace == G_MAXSIZE)
+                    last_value_whitespace = value.len;
+                nm_str_buf_append_c(&value, c);
+            }
+            break;
+
+        case VALUE_ESCAPE:
+            state = VALUE;
+            if (!nm_ascii_is_newline(c)) {
+                /* Escaped newlines we eat up entirely */
+                nm_str_buf_append_c(&value, c);
+            }
+            break;
+
+        case SINGLE_QUOTE_VALUE:
+            if (c == '\'')
+                state = PRE_VALUE;
+            else
+                nm_str_buf_append_c(&value, c);
+            break;
+
+        case DOUBLE_QUOTE_VALUE:
+            if (c == '"')
+                state = PRE_VALUE;
+            else if (c == '\\')
+                state = DOUBLE_QUOTE_VALUE_ESCAPE;
+            else
+                nm_str_buf_append_c(&value, c);
+            break;
+
+        case DOUBLE_QUOTE_VALUE_ESCAPE:
+            state = DOUBLE_QUOTE_VALUE;
+            if (strchr(SHELL_NEED_ESCAPE, c)) {
+                /* If this is a char that needs escaping, just unescape it. */
+                nm_str_buf_append_c(&value, c);
+            } else if (c != '\n') {
+                /* If other char than what needs escaping, keep the "\" in place, like the
+                 * real shell does. */
+                nm_str_buf_append_c(&value, '\\', c);
+            }
+            /* Escaped newlines (aka "continuation lines") are eaten up entirely */
+            break;
+
+        case COMMENT:
+            if (c == '\\')
+                state = COMMENT_ESCAPE;
+            else if (nm_ascii_is_newline(c)) {
+                state = PRE_KEY;
+                line++;
+            }
+            break;
+
+        case COMMENT_ESCAPE:
+            state = COMMENT;
+            break;
+        }
+    }
+
+    if (NM_IN_SET(state,
+                  PRE_VALUE,
+                  VALUE,
+                  VALUE_ESCAPE,
+                  SINGLE_QUOTE_VALUE,
+                  DOUBLE_QUOTE_VALUE,
+                  DOUBLE_QUOTE_VALUE_ESCAPE)) {
+        if (state == VALUE)
+            if (last_value_whitespace != G_MAXSIZE)
+                nm_str_buf_get_str_unsafe(&value)[last_value_whitespace] = 0;
+
+        /* strip trailing whitespace from key */
+        if (last_key_whitespace != G_MAXSIZE)
+            nm_str_buf_get_str_unsafe(&key)[last_key_whitespace] = 0;
+
+        r = push(line, nm_str_buf_get_str(&key), nm_str_buf_get_str(&value) ?: "", userdata);
+        if (r < 0)
+            return r;
+    }
+
+    return 0;
+}
+
+/*****************************************************************************/
+
+static int
+check_utf8ness_and_warn(const char *key, const char *value)
+{
+    /* Taken from systemd's check_utf8ness_and_warn()
+     * https://github.com/systemd/systemd/blob/6247128902ca71ee2ad406cf69af04ea389d3d27/src/basic/env-file.c#L273 */
+
+    if (!g_utf8_validate(key, -1, NULL))
+        return -EINVAL;
+
+    if (!g_utf8_validate(value, -1, NULL))
+        return -EINVAL;
+
+    return 0;
+}
+
+static int
+parse_env_file_push(unsigned line, const char *key, const char *value, void *userdata)
+{
+    const char *k;
+    va_list    *ap = userdata;
+    va_list     aq;
+    int         r;
+
+    r = check_utf8ness_and_warn(key, value);
+    if (r < 0)
+        return r;
+
+    va_copy(aq, *ap);
+
+    while ((k = va_arg(aq, const char *))) {
+        char **v;
+
+        v = va_arg(aq, char **);
+        if (nm_streq(key, k)) {
+            va_end(aq);
+            g_free(*v);
+            *v = g_strdup(value);
+            return 1;
+        }
+    }
+
+    va_end(aq);
+    return 0;
+}
+
+int
+nm_parse_env_filev(const char *contents, va_list ap)
+{
+    va_list aq;
+    int     r;
+
+    /* Copied from systemd's parse_env_filev().
+     * https://github.com/systemd/systemd/blob/6247128902ca71ee2ad406cf69af04ea389d3d27/src/basic/env-file.c#L333 */
+
+    va_copy(aq, ap);
+    r = nm_parse_env_file_full(contents, parse_env_file_push, &aq);
+    va_end(aq);
+    return r;
+}
+
+int
+nm_parse_env_file_sentinel(const char *contents, ...)
+{
+    va_list ap;
+    int     r;
+
+    /* Copied from systemd's parse_env_file_sentinel().
+     * https://github.com/systemd/systemd/blob/6247128902ca71ee2ad406cf69af04ea389d3d27/src/basic/env-file.c#L347 */
+
+    va_start(ap, contents);
+    r = nm_parse_env_filev(contents, ap);
+    va_end(ap);
+    return r;
+}
